#include <iostream>
template <typename Mat,     typename Vec,
	  typename Precond, typename Real>
int cg (const Mat &A, Vec &x, const Vec &b,
  const Precond &M, int maxiter, Real tol)
{
  Real normb = norm(b);
  if (normb == 0) { x = 0; return 0; }
  Vec r = b - A*x;
  Vec z = M.solve(r);
  Vec p = z;
  Real rho = dot(r, z);
  Real rho_prev = rho;
  std::clog << "pcg: 0 " << norm(r)/normb << std::endl;
  for (int i = 1; i < maxiter; i++) {
    Vec q = A*p;
    Real alpha = rho / dot(p, q);
    x += alpha * p;
    r -= alpha * q;
    Real resid = norm(r) / normb;
    std::clog << "pcg: "<< i << " " << resid << std::endl;
    if (resid <= tol) return 0;
    z = M.solve(r);
    rho = dot(r, z);
    Real beta = rho/rho_prev;
    rho_prev = rho;
    p = z + beta*p;
  }
  return 1;
}
