// valarray::size()
// valarray::resize(n,initval)

#include <valarray>
#include <iostream>
using namespace std;
class indeterminee : public string {
    public:
        indeterminee (const string& s = "X") : string(s) {}
};
template <typename T>
class polynome : public valarray<T> {
    public:
	polynome(const T& = T());
	polynome(const indeterminee&);
	polynome(const polynome<T>& q) : valarray<T>(q), X(q.X) {}
	polynome<T>& operator= (const polynome<T>& q) { 
		valarray<T>::resize (q.size());
		valarray<T>::operator=(q);
		X = q.X;
		return *this;
	}
	string symbol() const { return X; }
	int degre() const;
	polynome<T> operator+ (const polynome<T>& q) const;
	polynome<T> operator- (const polynome<T>& q) const;
	// friend polynome<T> operator* <> (const T&, const polynome<T>&);
	template <typename U> friend ostream& operator<< (ostream& os, const polynome<U>& p);
	void dump (string name, ostream& = cerr) const;
	void strip ();
    protected:
	indeterminee X;
};
template <typename T>
polynome<T>::polynome(const T& c0)
 : valarray<T>(c0,1), X()
{
}
template <typename T>
polynome<T>::polynome(const indeterminee& X1)
 : valarray<T>(1.0,2), X(X1)
{
    valarray<T>::operator[](0) = 0;
}
template <typename T>
int polynome<T>::degre() const
{
    return valarray<T>::size() -1;
}
template <typename T>
polynome<T>
polynome<T>::operator+ (const polynome<T>& q) const
{
    int sz_p = valarray<T>::size();
    int sz_q = q.size();
    int sz = (sz_p > sz_q) ? sz_p : sz_q;
    polynome<T> r;
    r.resize(sz, T());
    for (int i = 0; i < sz_p; i++) r[i] = valarray<T>::operator[](i);
    for (int i = 0; i < sz_q; i++) r[i] += q[i];
    r.strip();
    return r;
}
template <typename T>
polynome<T>
polynome<T>::operator- (const polynome<T>& q) const
{
    int sz_p = valarray<T>::size();
    int sz_q = q.size();
    int sz = (sz_p > sz_q) ? sz_p : sz_q;
    polynome<T> r;
    r.resize(sz, T());
    for (int i = 0; i < sz_p; i++) r[i] = valarray<T>::operator[](i);
    for (int i = 0; i < sz_q; i++) r[i] -= q[i];
    r.strip();
    return r;
}
template <typename T>
ostream&
operator<< (ostream& out, const polynome<T>& p)
{
    if (p.size() == 0) {
	out << "0"; 
	return out;
    }
    bool prem = true;
    for (int i = p.size()-1;  i >= 0; i--) {
	if (p[i] == 0) continue;
	if (!prem && p[i] > 0) out << "+";
	prem = false;
	if (i == 0) {
	    out << p[i];
	    continue;
	}
	if (p[i] == 1) {
	    ;
	} else if (p[i] == -1) {
	    out << "-";
	} else {
	    out << p[i] << "*";
        }
	// out << p.X;
	out << p.symbol();
	if (i == 1) continue;
	out << "^" << i;
    }
    return out;
}
template <typename T>
void
polynome<T>::dump (string name, ostream& err) const
{
    int sz_p = valarray<T>::size();
    for (int i = 0; i < sz_p; i++) {
	err << name << "[" << i << "] = " << valarray<T>::operator[](i) << endl;
    }
}
template <typename T>
polynome<T>
operator* (const T& c, const polynome<T>& p)
{
    polynome<T> q;
    if (c == T()) return q;
    q.resize(p.size(), T());
    for (size_t i = 0; i < q.size(); i++) q[i] = c*p[i];
    return q;
}
template <typename T>
polynome<T>
operator* (const polynome<T>& p, const polynome<T>& q)
{
    polynome<T> r;
    r.resize(p.size() + q.size() - 1, T());
    for (size_t i = 0; i < p.size(); i++) {
      for (size_t j = 0; j < q.size(); j++) {
	 r[i+j] += p[i]*q[j];
      }
    }
    return r;
}
template <typename T>
void
polynome<T>::strip ()
{
    int sz_p = valarray<T>::size();
    int imax = sz_p -1;
    for (; imax >= 0; imax--) {
	if (valarray<T>::operator[](imax) != T()) break;
    }
    if (imax+1 == sz_p) return;
    valarray<T> c(imax+1);
    for (int i = 0; i < imax+1; i++) c[i] = valarray<T>::operator[](i);
    valarray<T>::resize(imax+1);
    for (int i = 0; i < imax+1; i++) valarray<T>::operator[](i) = c[i];
}
template <typename T>
polynome<T>
pow(indeterminee X, int d)
{
    polynome<T> p;
    p.resize (d+1, T());
    p[d] = 1;
    return p;
}
template <typename T>
polynome<T>
div (const polynome<T>& a, const polynome<T>& b, 
     polynome<T>& q, polynome<T>& r)
{
    indeterminee X;
    q = polynome<T>();
    r = a;
    int d = 0;
    while (r.degre() != -1 && (d = r.degre() - b.degre()) >= 0) {
	polynome<T> t = r[r.degre()]/b[b.degre()] * pow<T>(X,d);
        q = q + t;
	r = r - b*t;
    }
    return r;
}
int main () {
    polynome<double> p = 3;
    indeterminee X;
    polynome<double> q = X;
    cout << p.degre() << endl;
    cout << q.degre() << endl;
    cout << p << endl;
    cout << q << endl;
    p.dump ("p");
    q.dump ("q");
    cout << p+q << endl;
    cout << (q+1)*(q+2) << endl;
    polynome<double> L[10];
    L[0] = 1;
    L[1] = X;
    for (int n = 1; n < 9; n++) {
	L[n+1] = (2.*n+1)/(n+1.)*polynome<double>(X)*L[n] - n/(n+1.)*L[n-1];
    }
    for (int n = 0; n < 10; n++) {
	cout << "L[" << n << "] = " << L[n] << endl;
    }
    polynome<double> z = (L[9])-L[9];
    cout << z.degre() << endl;
    cout << z << endl;
    z.dump("z");
    cout << pow<double>(X,6) << endl;
    polynome<double> x = X;
    polynome<double> a = 3.*x*x*x + x*x + x + 5;
    polynome<double> b = 5.*x*x - 3.*x + 1;
    polynome<double> r;
    div (a,b,q,r);
    cout << "q = " << q << endl;
    cout << "r = " << r << endl;
}

