// quaternions
#include <iostream>
#include <complex>
using namespace std;

// ================================================================
// question 7
// ================================================================

template <typename T>
class quaternion {
  public:
    quaternion (const T& a=0, const T& b=0, const T& c=0, const T& d=0) : z(a,b), w(c,d) {}
    quaternion (const complex<T>& z0, const complex<T>& w0) : z(z0), w(w0) {}
    quaternion (const quaternion<T>& h) : z(h.z), w(h.w) {}
    quaternion<T>& operator= (const quaternion<T>& h) { z = h.z; w = h.w; return *this; }
    friend quaternion<T> operator+ (const quaternion<T>& h, quaternion<T> m) {
	 quaternion<T> r;
	 r.z = h.z + m.z;
	 r.w = h.w + m.w;
	 return r;
    }
    friend quaternion<T> operator- (const quaternion<T>& h, quaternion<T> m) {
	 quaternion<T> r;
	 r.z = h.z - m.z;
	 r.w = h.w - m.w;
	 return r;
    }
    friend quaternion<T> operator* (const quaternion<T>& h1, quaternion<T> h2) {
	 quaternion<T> r;
	 r.z = h1.z*h2.z - h1.w*conj(h2.w);
	 r.w = h1.z*h2.w + h1.w*conj(h2.z);
	 return r;
    }
    friend quaternion<T> conj (const quaternion<T>& h) {
	 quaternion<T> r;
	 r.z = conj(h.z);
	 r.w = -h.w;
	 return r;
    }
    static T sqr(T x) { return x*x; }
    friend quaternion<T> abs (const quaternion<T>& h) {
	 return sqrt(sqr(abs(h.z)) + sqr(abs(h.w)));
    }
    friend quaternion<T> operator/ (const quaternion<T>& h1, quaternion<T> h2) {
	 quaternion<T> r = h1*conj(h2);
	 T deno = sqr(abs(h2.z)) + sqr(abs(h2.w));
	 r.z /= deno;
	 r.w /= deno;
	 return r;
    }
    friend ostream& operator << (ostream& out, const quaternion<T>& h) {
	 out << "(" << h.z << ", " << h.w << ")";
	 return out;
    }
  // data:
    complex<T> z, w;
};
// ================================================================
int main(int argc, char**argv) {
    quaternion<double> h1 (1,1,7,9);
    quaternion<double> h2 (1,-1,-7,-9);
    cout << "h1 = " << h1 << endl;
    cout << "h2 = " << h2 << endl;
    cout << "h1+h2 = " << h1+h2 << endl;
    cout << "h1-h2 = " << h1-h2 << endl;
    cout << "h1*h2 = " << h1*h2 << endl;
    cout << "h1/h2 = " << h1/h2 << endl;
    cout << "(h1/h2)*h2 = " << (h1/h2)*h2 << endl;
    return 0;
}
