// ---------------------------------------------------------------------------
// array with expression template
//  voir Vandevoorde & Josuttis, p. 328
//  TODO: enlever les T pour simplifier : T=double
// ---------------------------------------------------------------------------
#include <iostream> 
#include <functional> // plus, minus, multiplies, divides, modulus, negate

template <typename T>
class simple_array {
    public:
     	explicit simple_array(size_t sz = 0, const T& initval = T())
	  : data(new T[sz]), n(sz) {
	    for (size_t i = 0; i < n; i++) data[i] = initval;
	    std::cerr << "init "<< n << " data : "  << data[0] << std::endl;
	}
     	simple_array(const simple_array& b)
	  : data(new T[b.n]), n(b.n) {
	    for (size_t i = 0; i < n; i++) data[i] = b.data[i];
	    std::cerr << "copy "<< n << " data : "  << data[0] << std::endl;
	}
     	~simple_array() { delete [] data; }
     	simple_array<T>& operator= (simple_array<T>& b) {
	    if (&b == this) return *this;
	    if (n != b.n) {
     	      delete [] data; 
	      data = new T[b.n];
	      n    = b.n;
	    }
	    for (size_t i = 0; i < n; i++) data[i] = b.data[i];
	    std::cerr << "affect "<< n << " data : "  << data[0] << std::endl;
	    return *this;
	}
	size_t size() const { return n; }
	const T& operator[] (size_t i) const { return data[i]; }
	T& operator[] (size_t i) { return data[i]; }

	template <typename Op, typename U, typename B, typename C>
	friend class expr;
    protected:
	T* data;
	size_t n;
};
template <typename A>
class array_traits {
  public:
    typedef A const& reference_type;
};
template <typename Op, typename T, typename A, typename B>
class expr {
    public:
	template <typename T2, typename A2>
	friend class array;
	
	template <typename Op2, typename T2, typename A2, typename B2>
	friend class expr;
	
	expr(const A& a1, const B& b1) : op(), a(a1), b(b1) {}
	size_t size() const { return a->size(); }
	T operator[] (size_t i) const { return op(a[i],b[i]); }
    protected:
	Op op;
	typename array_traits<A>::reference_type a;
	typename array_traits<B>::reference_type b;
};
template <typename T>
class array_scalar {
    public:
	array_scalar(const T& v) : value(v) {}
	size_t size() const { return 0; }
	T operator[] (size_t) const { return value; }
    protected:
	T const& value;
};
template <typename T>
class array_traits<array_scalar<T> > {
  public:
    typedef array_scalar<T> reference_type;
};
template <typename T, typename A = simple_array<T> >
class array {
    public:
	explicit array (size_t n = 0, const T& initval = T()) : a(n,initval) {}
	template <typename B>
	array (const array<T,B>& b) : a(b.size()) {
	  for (size_t i = 0; i < b.size(); i++) 
	    a[i] = b[i];
        }
	template <typename B>
	array<T,A>& operator= (const array<T,B>& expr) {
	  for (size_t i = 0; i < a.size(); i++) 
	    a[i] = expr[i];
	  return *this;
	}
	size_t size() const { return a.size(); }
	T operator[] (size_t i) const { return a[i]; }
	T& operator[] (size_t i) { return a[i]; }

	template <typename Op, typename U, typename B, typename C>
	friend class expr;

	template <typename U, typename B>
	friend class array;

	template <typename B>
	const array<T,class expr<std::plus<T>,T,A,B> >
	operator+ (const array<T,B>& y) const {
            return array<T,expr<std::plus<T>,T,A,B> >(expr<std::plus<T>,T,A,B>(a,y.a));
        }
	template <typename B>
	const array<T,class expr<std::multiplies<T>,T,A,B> >
	operator* (const array<T,B>& y) const {
            return array<T,expr<std::multiplies<T>,T,A,B> >(expr<std::multiplies<T>,T,A,B>(a,y.a));
        }
	friend
	const array<T,class expr<std::multiplies<T>,T,array_scalar<T>,A> >
	operator* (const T& lambda, const array<T,A>& x) {
	    array_scalar<T> coef (lambda);
            typedef expr<std::multiplies<T>,T,array_scalar<T>,A> C;
            return array<T,C>(C(coef,x.a));
        }
    protected:
	array (const A& a0) : a(a0) {}
	A a;
};
#include <iostream>
using namespace std;
int main () {
  size_t n = 5;
  array<double> x(n, 1), y(n, 2), z(n, 5), t(n, 0);
  cerr << "before..." << endl;
  t = 1.2*x + y*z;
  cerr << "after..." << endl;
  for (size_t j = 0; j < z.size(); j++)
    cout << j << " " << t [j] << endl;
}
