#include "fd_energy_1d.h"
#include "fd_mass_1d.h"
#include "interpolate_1d.h"
#include "u_exact_singular.h"
#include "range.h"
#include "valarray_util.h"
#include <iostream>
using namespace std;
int main (int argc, char** argv) {
        size_t n = (argc > 1) ? atoi(argv[1]) : 11;
        double alpha = (argc > 3) ? atof(argv[3]) : 0.5;
        double beta  = (argc > 2) ? atof(argv[2]) : 2/alpha;
        valarray<double> x (n+1);
	for (size_t i = 0; i < n+1; i++)
	    x [i] = -1 + 2*chi (beta, 1.0*i/n);
	strid<double>    a (n-1);
        valarray<double> m (n-1);
	fd_energy_1d (x, a);
	fd_mass_1d   (x, m);
	strid<double>    fact (n-1);
	factorize (a, fact);
        valarray<double> pi_h_f = interpolate (x, f(alpha));
	valarray<double> b = m * valarray<double>(pi_h_f [range(1,n-1)]);
        valarray<double> uh (0.0, n+1);
	uh [range(1,n-1)] = fact.solve (b);
        valarray<double> pi_h_u = interpolate (x, u(alpha));
        valarray<double> u_err = valarray<double>(pi_h_u - uh)[range(1,n-1)];
        valarray<double> uerr_linf = abs(u_err);
	cerr << "alpha    " << alpha << endl
	     << "beta     " << beta << endl
	     << "n        " << n << endl
             << "err_l2   " << sqrt(dot (u_err, m*u_err)) << endl
             << "err_linf " << uerr_linf.max() << endl
             << "err_h1   " << sqrt(dot (u_err, a*u_err)) << endl;
}
