#ifndef _COURS_JACOBI_H
#define _COURS_JACOBI_H
//
// computes the Jacobi (a,b) polynomial 'p' and its derivative 'dp_dx'
// at x for order N, N-1 and N-2 :
//
//	(p, dp_dx) 		: degree N
//	(p, dp_dx)_prev 	: degree N-1
//	(p, dp_dx)_prev2 	: degree N-2
//
// Legendre   = Jacobi(0,0)
// Chebyschev = Jacobi(-0.5,-0.5) => explicit formulae
//
template <typename Size, typename T>
void
jacobi (const T& a, const T& b, Size N, const T& x,
	    T& p, 	T& dp_dx, 
	    T& p_prev,  T& dp_dx_prev, 
	    T& p_prev2, T& dp_dx_prev2)
{
	p     = 1;
	dp_dx = 0;
	if (N == 0) return;
	p_prev     = p;
	dp_dx_prev = dp_dx;
	p     = (1 + a)*x;
	dp_dx =  1 + a;
	if (N == 1) return;
	for (Size k = 2; k <= N; k++) {
	    T a1 = 2*k*(k + a + b)*(2*k + a + b - 2);
	    T a2 = (2*k + a + b - 1)*(a*a - b*b);
	    T b3 =  2*k + a + b - 2;
	    T a3 = b3*(b3 + 1)*(b3 + 2);
	    T a4 = 2*(k + a - 1)*(k + b - 1)*(2*k + a + b);
	    T p_next     = ((a2 + a3*x)*p     - a4*p_prev)/a1;
	    T dp_dx_next = ((a2 + a3*x)*dp_dx - a4*dp_dx_prev + a3*p)/a1;

	    p_prev2 = p_prev;
	    dp_dx_prev2 = dp_dx_prev;
	    p_prev = p;
	    dp_dx_prev = dp_dx;
	    p = p_next;
	    dp_dx = dp_dx_next;
	}
}
#endif // _COURS_JACOBI_H
