#ifndef _COURS_LAGRANGE_DERIVATIVE_CHECK_H
#define _COURS_LAGRANGE_DERIVATIVE_CHECK_H
//
// when degree(p) <= N, the formulae is exact:
// 	dp_dx (x[i]) = sum_{j=0..N} d(i,j) * p(x[j]),  i=0..N
//
// => check it for p(x) = x^n,  n=0..N
//
#include "lagrange_derivative.h"
#include <iterator>
#include <cmath>

template <typename Iterator1, typename Iterator2, typename Matrix, typename Size>
typename std::iterator_traits<Iterator1>::value_type
lagrange_derivative_check (Iterator1 x, Iterator2 w, const Matrix& d, Size N) {
	typedef typename std::iterator_traits<Iterator1>::value_type T;
	T err = 0;
	for (Size n = 0; n <= N; n++) {
	    for (Size i = 0; i <= N; i++) {
	      T dp_dx = 0;
	      for (Size j = 0; j <= N; j++)
		dp_dx += d(i,j)*std::pow (x[j], int(n));
	      T exact = (n == 0) ? 0 : n*std::pow (x[i],int(n-1));
	      T err_n_i = std::abs (dp_dx - exact);
	      err = std::max (err, err_n_i);
	    }
	}
	return err;
}
#endif // _COURS_LAGRANGE_DERIVATIVE_CHECK_H
