#!/bin/sh

use_ps2epsi=false
if test $#  -eq 0; then
	echo "usage: $0 <basename>[.psplot|.plot]" >&2
	echo "     output: <basename>.ps" >&2
	exit 1
fi
x=$1
x=`expr ${x} : '\(.*\)\.psplot' \| ${x}`
x=`expr ${x} : '\(.*\)\.plot'   \| ${x}`
if test -f ${x}.psplot -a -f ${x}.plot; then
	echo "$0: both ${x}.psplot and ${x}.plot exists" >&2
	exit 1
fi
if test ! -f ${x}.psplot -a ! -f ${x}.plot; then
	echo "$0: file ${x}.[psplot|plot] not found" >&2
	exit 1
fi
if test -f ${x}.psplot; then
    	input=${x}.psplot
else
    	input=${x}.plot
fi

echo "! gnuplot ${input}"
gnuplot ${input}
status=$?
if test $status -ne 0; then
	echo "$0: gnuplot exit with status = $status" >&2
	exit $status
fi
cat > ${x}-wrap.tex << EOF
\documentclass{article}
\usepackage{amssymb}
\begin{document}
\thispagestyle{empty}
\input{${x}.tex}
\end{document}
EOF
echo "! file \"${x}-wrap.tex\" created"

echo "! latex ${x}-wrap.tex"
latex ${x}-wrap.tex

echo "! dvips -E ${x}-wrap.dvi -o ${x}-wrap.ps"
dvips -E ${x}-wrap.dvi -o ${x}-wrap.ps

if ${use_ps2epsi}; then
  echo "! ps2epsi ${x}-wrap.ps ${x}.ps"
  ps2epsi ${x}-wrap.ps ${x}.ps
  status=$?
else
  mv ${x}-wrap.ps ${x}.ps
  status=0
fi
rm -f ${x}.tex ${x}
rm -f ${x}-wrap.tex ${x}-wrap.dvi ${x}-wrap.aux ${x}-wrap.log ${x}-wrap.ps

exit $status
