#include "legendre_gauss_lobatto.h"
#include "lagrange_derivative.h"
#include "spectral_assembly_energy.h"
#include "matrix.h"
#include "strid.h"
#include "fd_energy_1d.h"
#include "range.h"
#include "pcg.h"
#include <limits>
template <typename Size, typename Array>
void spectral_dirichlet_pcg (const Array& zeta, const Array& w, const Array& b, Size N,
		Array& uh) {
	typedef typename Array::value_type T;
        Array LN = interpolate (zeta, legendre<T>(N));
	matrix_store<T> ds (N+1,N+1);
	matrix_store<T> as (N-1,N-1);
	lagrange_derivative (zeta, LN, N, ds);
	spectral_assembly_energy (ds, w, as, N);
	matrix<T> a;
	convert (as, a);
        Array x (N-1);
        strid<double> finite_difference (N-1);
        fd_energy_1d (zeta, finite_difference);
        finite_difference.factorize ();
	T tol = 10*std::numeric_limits<T>::epsilon();
	if (pcg (a, x, b, finite_difference, 2*N, tol) != 0) {
	    std::cerr << "pcg: precision not reached\n";
	    exit (1);
	}
        uh [0] = uh [N] = 0;
	uh [range (1,N-1)] = x;
}
