#include "legendre_gauss_lobatto.h"
#include "legendre.h"
#include "interpolate_1d.h"
#include "lagrange_derivative.h"
#include "dense_matrix.h"
#include "strid.h"
#include "fd_energy_1d.h"
#include "itos.h"
#include "valarray_util.h"
#include <string>
#include <fstream>
#include <limits>
using namespace std;
int main (int argc, char**argv) {
	size_t N     = (argc > 1) ? atoi(argv[1]) : 10;
	string root  = (argc > 2) ?      argv[2]  : "a";
	string name =  root + "_" + itos(N);
        valarray<double> zeta (N+1);
        valarray<double> w (N+1);
        legendre_gauss_lobatto (zeta, w);
        valarray<double> LN = interpolate (zeta, legendre<double>(N));
	dense_matrix<double> d (N+1,N+1);
        lagrange_derivative (zeta, LN, d);
        strid<double> M (N-1);
        fd_energy_1d (zeta, M);

	string weight_mm = name + "_weight.v";
	ofstream wf (weight_mm.c_str());
	cerr << "! file \"" << weight_mm << "\" created\n";
	wf.precision (std::numeric_limits<double>::digits10);
	wf << w;
	wf.close();

	string derivative_mm = name + "_derivative.mm";
	ofstream df (derivative_mm.c_str());
	cerr << "! file \"" << derivative_mm << "\" created\n";
	df.precision (std::numeric_limits<double>::digits10);
	df << d;
	df.close();

	string precond_mm = name + "_precond.mm";
	ofstream mf (precond_mm.c_str());
	cerr << "! file \"" << precond_mm << "\" created\n";
	mf.precision (std::numeric_limits<double>::digits10);
	mf << M;
	mf.close();
}
