#include "matrix.h"
template <typename T>
class ssor : public matrix<T> {
  public:
    ssor(const matrix<T>& a, const T& w = T(1));
    std::valarray<T> solve (const std::valarray<T>&) const;
  protected:
    std::valarray<T> d;
    T w;
};
template <typename T>
ssor<T>::ssor(const matrix<T>& a, const T& w0)
: matrix<T>(),
  d(T(), std::min(a.nrow(),a.ncol())),
  w(w0)
{
  matrix<T>::resize(a.nrow(),a.ncol(), (a.nnz() - std::min(a.nrow(),a.ncol()))/2);
  size_t q = 0;
  matrix<T>::ptr[0] = 0;
  for (size_t i = 0; i < d.size(); i++) {
    for (size_t p = a.ptr[i]; p < a.ptr[i+1]; p++) {
      if (i == a.idx[p]) {
        d[i] = a.val[p];
      } else if (i > a.idx[p]) {
        matrix<T>::idx[q] = a.idx[p];
        matrix<T>::val[q] = a.val[p];
	q++;
      }
    }
    matrix<T>::ptr[i+1] = q;
  }
}
template <typename T>
std::valarray<T> ssor<T>::solve (const std::valarray<T>& b) const
{
  // solve : (D/w+L)*x3 = b
  std::valarray<T> x = b;
  for (size_t i = 0; i < b.size(); i++) {
    for (size_t p = matrix<T>::ptr[i]; p < matrix<T>::ptr[i+1]; p++)
      x[i] -= matrix<T>::val[p] * x[matrix<T>::idx[p]];
    x[i] /= d[i]/w;
  }
  // solve : ((2-w)*D/w)^(-1)*x2 = x3
  for (size_t i = 0; i < b.size(); i++)
    x[i] *= (2-w)*d[i]/w;

  // solve : (D/w+L)^T*x = x2
  for (size_t i = b.size(); i > 0; i--) {
    x[i-1] /= d[i-1]/w;
    for (size_t p = matrix<T>::ptr[i-1]; p < matrix<T>::ptr[i]; p++)
      x[matrix<T>::idx[p]] -= matrix<T>::val[p] * x[i-1];
  }
  return x;
}
